def find_closest(v, array):
    # Finds the closest value in an array for one given
    # value.
    import numpy as np

    diff = abs(array - v)
    index = np.argmin(diff)
    return index
    
def HMS2timestamp(hours,minutes,seconds,day_later=False):
    # Converts arrival times in hours, minutes and seconds 
    # to a timestamp based on the amount of seconds since
    # the start of the day of the earliest arrival
    ts = hours*3600 + minutes*60 + seconds
    if day_later:
        ts = ts + 24*3600
    return ts

def timestamp2HMS(timestamp, day_later = False):
    # Converts timestamps back to hours, minuts and
    # seconds.
    if day_later:
        timestamp = timestamp - 24*3600
    H = timestamp//3600
    rest1 = timestamp - H*3600
    M = rest1//60
    S = rest1 - M*60
    return [H,M,S]

def geocen(lat):
    # Corrects the latitude for ellipticity

    import numpy as np
    pi = np.pi

    r2d = 180.0/pi
    d2r = pi/180.0

    R = 6378.160
    ellip = 1.0/298.25
    b = R*(1.0-ellip)
    gfact = (b*b)/(R*R)
    glat = r2d*np.arctan(gfact*(np.sin(lat*d2r)/np.cos(lat*d2r)))
    return glat

def distaz(la1, lo1, la2, lo2):
    # Finds the epicentral distance, azimuth and backazimuth
    # for two locations given in latitude and longitude.

    import numpy as np
    pi = np.pi

    if la1 == la2 and lo1 == lo2:
        gcarc, az, baz = 0.0, 0.0, 0.0
    else:
        r2d = 180/pi
        d2r = pi/180
    
        glat1 = geocen(la1)*d2r
        clat1 = np.cos(glat1)
        slat1 = np.sin(glat1)
        
        glat2 = geocen(la2)*d2r
        clat2 = np.cos(glat2)
        slat2 = np.sin(glat2)
        
        rdlon = (lo2 - lo1)*d2r
        cdlon = np.cos(rdlon)
        sdlon = np.sin(rdlon)
        
        cdel = slat1 * slat2 + clat1 * clat2 * cdlon
        if cdel > 1.0:
            cdel = 1.0
        if cdel < -1.0:
            cdel = -1.0

        yazi = sdlon * clat2
        xazi = clat1*slat2 - slat1*clat2*cdlon
        ybaz = -sdlon * clat1
        xbaz = clat2*slat1 - slat2*clat1*cdlon

        gcarc = r2d * np.arccos(cdel)
        az = r2d * np.arctan2(yazi,xazi)
        baz = r2d * np.arctan2(ybaz,xbaz)

        if az < 0.0: 
            az = az + 360.0
        if baz < 0.0:
            baz = baz + 360.0
    return gcarc, az, baz

def ttime(delta, d, Pdist, Pdepth, Pgrid):
    # Finds the correct traveltime value for a given depth and 
    # epicentral distance, by interpolating a given grid of 
    # traveltimes based on a 1D-earth model. It also provides
    # the derivatives with respect to both axis (depth and 
    # distance) at this point. NOT FINISHED
    import numpy as np
    from scipy import interpolate

    #initial setting
    nmax_dis = 11
    nmax_dep = 5

    dis_range = nmax_dis//2
    dep_range = nmax_dep//2
    #find closest values in table
    Idis = find_closest(delta, Pdist)
    Idep = find_closest(d, Pdepth)
    
    #set ranges of considered values
    Idis_min = max(Idis - dis_range, 0)
    Idis_max = min(Idis_min + nmax_dis, len(Pdist))
    if Idis_max == len(Pdist):
        Idis_min = len(Pdist) - nmax_dis
    
    Idep_min = max(Idep - dep_range, 0)
    Idep_max = min(Idep_min + nmax_dep, len(Pdepth))
    if Idep_max == len(Pdepth):
        Idep_min = len(Pdepth) - nmax_dep
    
    X = Pdist[Idis_min:Idis_max]
    Z = Pdepth[Idep_min:Idep_max]
    t = Pgrid[Idep_min:Idep_max,Idis_min:Idis_max]
    
    #interpolate using the 2D-spline method from scipy
    X_,Z_ = np.meshgrid(X,Z)
    X1,Z1,t1 = X_.flatten(),Z_.flatten(),t.flatten()
    cors = np.array([X1,Z1]).transpose()
    
    grid_x, grid_z = np.mgrid[X[0]:X[-1]:110j, Z[0]:Z[-1]:50j]                               
    Ts = interpolate.griddata(cors,t1,(grid_x,grid_z),method = 'cubic').transpose()
    
    xi = find_closest(delta, grid_x[:,0])
    zi = find_closest(d,grid_z[0,:])
     
    #find predicted value and the distance and depth derivative at that point
    t_pred = Ts[zi,xi]
    DtDdep = (Ts[zi+1,xi] - Ts[zi-1,xi])/(grid_z[0,zi+1] - grid_z[0,zi-1])
    DtDdis = (Ts[zi,xi+1] - Ts[zi,xi-1])/(grid_x[xi+1,0] - grid_x[xi-1,0])

    return t_pred, DtDdep, DtDdis
